/*
SerialDisplay.h - Library for Sparkfunk serialdisplay
Version 2.4

by Cordier Yves 
release :
V1.1 : 01/2015
V2.0 : 11/04/2016 adapted to ARM Arduino boards the 11/04/2016 (tested on due and uno)
V2.01 : 17/04/2016 Clreoln() added
V2.02 : 24/04/2016 Clreoln() debugged
V2.1  : 14/05/2016 harward serial connection allowed
V2.11 : 10/05/2017 correction to fix bug on due
V2.12 : 14/05/2017 type unsigned long long and long long added to print functions
V2.13 : 15/05/2017 correction of bug on V2.12 about long long 
V2.14 : 16/05/2017 simplification and producing more compact code
				added possibility of printing zeros before significant figures
				in print(n,base) could be write print (n,base,true) to do so;
V2.2  : 18/05/2017 rewriting of almost half of the librarie using C++ facilities
					accepting strings with \ characteres
					\n \t \v \b \r \f \a \\ \' \" \? \0
v2.21 : 19/05/2017 using pgm memory to store large arrays by /avr/pgmspace.h librairie
                   Correction de la commande Begin qui possède deux parametres (vitesse, reinit)
				   vitesse est de 9600 bds par defaut et reinit est true par defaut
				   en mode final (appli stabilisée), il vaut mieux utiliser reinit à false
v2.22 :	19/10/2017  bug of cleroln fixed		
v2.23 : added functions to sleep and to awake the display when this one's power supply is connected to an output pin 	SerialDisplay.h - Library for Sparkfunk serialdisplay
v2.24 : minors bugs corrected to avoid warning's compiler
v2.3  :	23/03/2020 add scientific mode for float and double printing
v2.4  : 23/06/2022 reorganisation de la librairie pour la V2.0 de l'IdE Arduino 
                   correction de bugs mineurs
				   
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
/* Cette librairie permet l'utilisation de sorties séries sur une broche
   par l'utilisation d'un objet de type SerialOut
   exemple :
   SerialOut ma_serie(3); // ouvre une sortie serie sur la broche 3
      
   ..
   ma_serie.begin(9600); // regle la vitesse de transmission à 9600 bauds 
                         // 300, 600, 1200,2400, 4800, 9600, 14400, (19200 38400 non testé)

   ma_serie.write(10); // envoi le caractère 10 0x0A au sérialisateur
   
   l'objet SerialDisplay permet de communiquer avec un afficheur de type 
   sparkfunk seriel
   exemple :
   
   SerialDisplay visu=SerialDisplay(5); // attache un afficheur à la borne 5
   // ou
   SerialDisplay visu=SerialDisplay(Serial) // attache un afficheur à la broche 1 par le serialisateur matériel Serial
   // Serial, Serial1, Serial2, Serial3 pour Mega due ...
   // Serial  Tx : 1   Rx : 0
   // Serial1 Tx : 18  Rx : 19
   // Serial2 Tx : 16  Rx : 17
   // Serial3 Tx : 14  Rx : 15
   ..
   visu.begin(9600); mets cet afficheur en 9600 bauds, 
   // vitesses autorisées : 2400, 4800, 9600, 14400, 19200, 38400
   
   visu.print("hello"); // affiche hello à partir du curseur
   visu.print(data); permet d'afficher tout type de données
   
    void print(char);
	void print(const char[]);
	void print(String);
	void print(byte);
	void print(int);
	void print(unsigned int);
	void print(long);
	void print(unsigned long);
	void print(long long);
	void print(unsigned long long);
	void print(byte, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(int, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(unsigned int, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(long long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(unsigned long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(unsigned long long, int=2,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut 
	void print(float , int=2); // impression avec int digits derriere la virgule 2 par defaut
	void print(double , int=2); // impression avec int digits derriere la virgule 2 par defaut
		
	// les memes procedures avec passage à la ligne
	void println(char);
	void println(const char[]);
	void println(String);
	void println(byte);
	void println(int);
	void println(unsigned int);
	void println(long);
	void println(unsigned long);
	void println(long long);
	void println(unsigned long long);
	void println(byte, int,boolean=false); 
	void println(int, int,boolean=false); 
	void println(unsigned int, int,boolean=false); 
	void println(long, int,boolean=false); 
	void println(long long, int,boolean=false); 
	void println(unsigned long, int,boolean=false); 
	void println(unsigned long long, int,boolean=false); 
	void println(float , int=2, scienti=false); 
	void println(double , int=2,scienti=false); 
	void println(void);
    
	
	void Clrscr(); //efface l'écran
	void CursRight(); //bouge le curseur vers la droite
	void CursLeft();  //bouge le curseur vers la gauche
	void ScrollRight(); //bouge tout l'écran vers la droite
	void ScrollLeft(); //bouge tout l'écran vers la gauche
	void Displayon(); //affiche on
	void Displayoff(); //affiche off
	void Underlineon(); //souligne le curseur
	void Underlineoff(); //curseur muet
	void Blinkingon(); //fait clignotter le curseur
	void Blinkingoff(); //arrete le clignottement
	void Posxy(byte x, byte y); //positionne le curseur en x y (le coin en haut à gauche est à (1,1)
	void Clreoln(); // efface la fin de la ligne en cours et passe à la ligne suivante
	
	void Backlight(byte i); //fait varier la puissance lumineuse i va de 1 à 31
   
   Attention !!
   avec int i=-10;
   print(i) =====> -10 Base 10 par defaut avec les conventions de stockage du signe
   print(i,10) =====> 65526 ; le fait de préciser la base implique que l'on regarde 
                            une quantite positive et donc que le bit de signe 
							cree des problemes d'interpretation
   int j=10;
   print(i) =======> 10
   print (i,10) ====> 10
   print (i,10,true) ===>00010 ; la capacité totale d'un int peut se decrire sous forme de 5
								digits en base 10 ici on affiche les digits non significatifs
						   
 */
#define _progmem // on utilise la bibliotheque pgmspace.h pour mettre en 
// memoire programme des tableaux de données

#ifndef SerialDisplay_h
#define SerialDisplay_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include <inttypes.h>
#include <avr/interrupt.h>

#ifdef _progmem
#include <avr/pgmspace.h>
#endif

#define Max_X 16 // Nb de caractere par lignes 16 ou 20
#define Max_Y 2 // Nb de lignes de caracterees 4 ou 2

#define Version "V2.4"

class SerialOut
{
// private:
protected: // ces attributs seront visibles dans les classes derivés mais non dans le pgm principal
	HardwareSerial *_serial;
	boolean hardSerial= false;
	boolean serialSet = false;
	uint8_t transmitPin;
	long baudRate ;
	float correctionAttente=0;
	long bitPeriod;
	long delaiEntreBits;
	void setSerial(); // mise en place des constantes de transmission en fonction de la vitesse
	unsigned long deltaT(unsigned long T1,unsigned long T2);
 // le delay en micro secondes pour une sortie sur broche

	
public:

// Les constructeurs
	SerialOut(); // Le constructeur par defaut
	SerialOut(uint8_t); // defini la broche de connection de la sortie serie
	SerialOut(HardwareSerial *ser); // defini la sortie serie materiel
	
	void begin(long); // defini la vitesse de communication

	void write(char); // la clef de voute du systeme envoi un caractère sur la voie série
		
};



class SerialDisplay : public SerialOut
{
private:
	
	
	boolean traiteCaractereSpeciaux(char s); // traitement des caractères spéciaux d'afichage du style 
	// \n \t \v \b \r \f \a \\ \? \' \" \0
	int cursX; // Le curseur logiciel cursX de 0.. _Max_X-1
	// position haut gauche (0,0)
	int _Max_X ; // Nb de caractères sur x
	int _Max_Y ; // Le nombre de lignes
	void printNumber(unsigned long long,byte, byte =10,boolean=false); // affichage d'un nombre entier
	// premier parametre : le nombre
	// deuxieme parametre : le nombre de byte de codage
	// troisieme parametre : la base
	// quatrieme parametre : affichage des 0 avant chiffre significatig I.E. : 0123 ou 123
	// Procedure à ne pas utiliser en direct !!
		int brocheOn =-1;
	// La broche qui allimente l'afficher (-1 : aucune)
	void printFloat(double , uint8_t ); // affichage d'un flotant
	void set_size_lcd();
	int powerPin=-1;
public:
// Les constructeurs
    SerialDisplay() ; // defini la sortie materielle
	SerialDisplay(HardwareSerial *ser); // defini la sortie materielle
	SerialDisplay(HardwareSerial *ser,uint8_t,uint8_t);// defini la serie materielle de connection de l'afficheur ainsi que la taille de l'afficheur
	                                        // broche, tx , ty ; tx vaut 16 ou 20 ty vaut 2 ou 4
	SerialDisplay(byte tx); // defini la broche de connection de l'afficheur par defaut un afficheur de 16*2
	SerialDisplay(uint8_t,uint8_t,uint8_t);// defini la broche de connection de l'afficheur ainsi que la taille de l'afficheur
	                                        // broche, tx , ty ; tx vaut 16 ou 20 ty vaut 2 ou 4
	void begin(long vitesse=9600,boolean reinit=true); // initialise l'afficheur (vitesse de communication 9600 par defaut et 
	                                                   // la reinitialisation
													   // en mode mise au point avec du matériel nopn stabilisé
													   // on utilise reinit à true
													   // en mode final, le matériel étant stabilisé, on passe à false
	void awake(long speed); // allume l'afficheur et l'initialise si une broche de connection puissnace a été déclarée
	void sleep(); // etteint l'affichier si une broche de connection puissance a été déclarée
	void setPowerPin(int p); // set the powerpin
	
	// void write(char); // la clef de voute du systeme envoi un caractère sur la voie série défini dans la classe mêre
	void print(char);
	void print(const char[]);
	void print(String);
	void print(byte);
	void print(int);
	void print(unsigned int);
	void print(long);
	void print(unsigned long);
	void print(long long);
	void print(unsigned long long);
	void print(byte, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(int, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(unsigned int, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(long long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(unsigned long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void print(unsigned long long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut 
	void print(double , int=2,boolean scienti=false); // impression avec int digits derriere la virgule 2 par defaut, mode scientifique non par defaut
	void print(float , int=2, boolean scienti=false ); // impression avec int digits derriere la virgule 2 par defaut, mode scientifique non par defaut
		
	// les memes procedures avec passage à la ligne
	void println(char);
	void println(const char[]);
	void println(String);
	void println(byte);
	void println(int);
	void println(unsigned int);
	void println(long);
	void println(unsigned long);
	void println(long long);
	void println(unsigned long long);
	void println(byte, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void println(int, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void println(unsigned int, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void println(long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void println(long long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void println(unsigned long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut
	void println(unsigned long long, int,boolean=false); // impression dans une base donnée int 0, 2..16 zero non significatifs non affiches par defaut 
	void println(double , int=2,boolean scienti=false); // impression avec int digits derriere la virgule 2 par defaut, mode scientifique non par defaut
	void println(float , int=2,boolean scienti=false); // impression avec int digits derriere la virgule 2 par defaut, mode scientifique non par defaut
	void println(void);
	
	void Clrscr(); //efface l'écran
	void CursRight(); //bouge le curseur vers la droite
	void CursLeft();  //bouge le curseur vers la gauche
	void ScrollRight(); //bouge tout l'écran vers la droite
	void ScrollLeft(); //bouge tout l'écran vers la gauche
	void Displayon(); //affiche on
	void Displayoff(); //affiche off
	void Underlineon(); //souligne le curseur
	void Underlineoff(); //curseur muet
	void Blinkingon(); //fait clignotter le curseur
	void Blinkingoff(); //arrete le clignottement
	void Posxy(byte x, byte y); //positionne le curseur en x y (le coin en haut à gauche est à (1,1)
	void Clreoln(); // efface la fin de la ligne en cours et passe à la ligne suivante
	void Backlight(byte i); //fait varier la puissance lumineuse i va de 1 à 31
	private:
	void printScienti(double d, byte dec); // impression format scientifique
	
	
};

#endif

